import { first } from 'rxjs/operators';
import { atom } from 'jotai';

function atomWithObservable(createObservable) {
  const observableResultAtom = atom((get) => {
    const observable = createObservable(get);
    let resolve = null;
    const resultAtom = atom(new Promise((r) => {
      resolve = r;
    }));
    resultAtom.scope = observableAtom.scope;
    let setResult = () => {
      throw new Error("setting data without mount");
    };
    const dataListener = (data) => {
      if (resolve) {
        resolve({ data });
        resolve = null;
      } else {
        setResult({ data });
      }
    };
    const errorListener = (error) => {
      if (resolve) {
        resolve({ error });
        resolve = null;
      } else {
        setResult({ error });
      }
    };
    observable.pipe(first()).toPromise().then(dataListener).catch(errorListener);
    resultAtom.onMount = (update) => {
      setResult = update;
      const subscription = observable.subscribe(dataListener, errorListener);
      return () => subscription.unsubscribe();
    };
    return { resultAtom, observable };
  });
  const observableAtom = atom((get) => {
    observableResultAtom.scope = observableAtom.scope;
    const { resultAtom } = get(observableResultAtom);
    const result = get(resultAtom);
    if ("error" in result) {
      throw result.error;
    }
    return result.data;
  }, (get, _set, data) => {
    observableResultAtom.scope = observableAtom.scope;
    const { observable } = get(observableResultAtom);
    if ("next" in observable) {
      observable.next(data);
    } else {
      throw new Error("observable is not subject");
    }
  });
  return observableAtom;
}

export { atomWithObservable };
