'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var operators = require('rxjs/operators');
var jotai = require('jotai');

function atomWithObservable(createObservable) {
  var observableResultAtom = jotai.atom(function (get) {
    var observable = createObservable(get);
    var resolve = null;
    var resultAtom = jotai.atom(new Promise(function (r) {
      resolve = r;
    }));
    resultAtom.scope = observableAtom.scope;

    var setResult = function setResult() {
      throw new Error('setting data without mount');
    };

    var dataListener = function dataListener(data) {
      if (resolve) {
        resolve({
          data: data
        });
        resolve = null;
      } else {
        setResult({
          data: data
        });
      }
    };

    var errorListener = function errorListener(error) {
      if (resolve) {
        resolve({
          error: error
        });
        resolve = null;
      } else {
        setResult({
          error: error
        });
      }
    };

    observable.pipe(operators.first()).toPromise().then(dataListener).catch(errorListener);

    resultAtom.onMount = function (update) {
      setResult = update;
      var subscription = observable.subscribe(dataListener, errorListener);
      return function () {
        return subscription.unsubscribe();
      };
    };

    return {
      resultAtom: resultAtom,
      observable: observable
    };
  });
  var observableAtom = jotai.atom(function (get) {
    observableResultAtom.scope = observableAtom.scope;

    var _get = get(observableResultAtom),
        resultAtom = _get.resultAtom;

    var result = get(resultAtom);

    if ('error' in result) {
      throw result.error;
    }

    return result.data;
  }, function (get, _set, data) {
    observableResultAtom.scope = observableAtom.scope;

    var _get2 = get(observableResultAtom),
        observable = _get2.observable;

    if ('next' in observable) {
      observable.next(data);
    } else {
      throw new Error('observable is not subject');
    }
  });
  return observableAtom;
}

exports.atomWithObservable = atomWithObservable;
